@@ -10,6 +10,7 @@ from devpi_common.archive import zip_dir
 from devpi_common.types import CompareMixin
 from .main import HTTPReply, set_devpi_auth_header
 
+
 def main(hub, args):
     # for now we use distutils/setup.py for register/upload commands.
 
@@ -19,7 +20,8 @@ def main(hub, args):
 
     current = hub.require_valid_current_with_index()
     if not args.index and not current.pypisubmit:
-        hub.fatal("no pypisubmit endpoint available for: %s" % current.index)
+        hub.fatal("The current index %s does not support upload."
+            "\nMost likely, it is a mirror." % current.index)
 
     if args.path:
         return main_fromfiles(hub, args)
@@ -51,6 +53,7 @@ def main(hub, args):
         uploader.pypisubmit = hub.current.get_index_url(args.index).url
     uploader.do_upload_paths(archives)
 
+
 def filter_latest(path_pkginfo):
     name_version_path = {}
     for archivepath, pkginfo in path_pkginfo.items():
@@ -64,6 +67,7 @@ def filter_latest(path_pkginfo):
         retval[x[2]] = x[1]
     return retval
 
+
 def main_fromfiles(hub, args):
     paths = []
     for p in args.path:
@@ -79,6 +83,7 @@ def main_fromfiles(hub, args):
         uploader.pypisubmit = hub.current.get_index_url(args.index).url
     uploader.do_upload_paths(paths)
 
+
 class Uploader:
     def __init__(self, hub, args, name_version=None):
         self.hub = hub
@@ -115,7 +120,6 @@ class Uploader:
         for archivepath, pkginfo in doczip2pkginfo.items():
             self.upload_doc(archivepath, pkginfo)
 
-
     def upload_doc(self, path, pkginfo):
         if self.name_version:
             (name, version) = self.name_version
@@ -179,8 +183,11 @@ class Uploader:
         meta["pyversion"], meta["filetype"] = pyver
         self.post("file_upload", path, meta=meta)
 
+
 # taken from devpi-server/extpypi.py
 ALLOWED_ARCHIVE_EXTS = ".egg .whl .tar.gz .tar.bz2 .tar .tgz .zip".split()
+
+
 def get_archive_files(path):
     if path.isfile():
         yield path
@@ -272,13 +279,19 @@ def find_parent_subpath(startpath, relpath, raising=True):
 class Checkout:
     def __init__(self, hub, setupdir, hasvcs=None, setupdir_only=None):
         self.hub = hub
+        self.cm_ui = None
+        if hasattr(check_manifest, 'UI'):
+            self.cm_ui = check_manifest.UI()
         assert setupdir.join("setup.py").check(), setupdir
         hasvcs = not hasvcs and not hub.args.novcs
         setupdir_only = bool(setupdir_only or hub.args.setupdironly)
         if hasvcs:
             with setupdir.as_cwd():
                 try:
-                    hasvcs = check_manifest.detect_vcs().metadata_name
+                    if self.cm_ui:
+                        hasvcs = check_manifest.detect_vcs(self.cm_ui).metadata_name
+                    else:
+                        hasvcs = check_manifest.detect_vcs().metadata_name
                 except check_manifest.Failure:
                     hasvcs = None
                 else:
@@ -300,11 +313,18 @@ class Checkout:
         if not self.hasvcs:
             return Exported(self.hub, self.setupdir, self.setupdir)
         with self.rootpath.as_cwd():
-            files = check_manifest.get_vcs_files()
+            if self.cm_ui:
+                files = check_manifest.get_vcs_files(self.cm_ui)
+            else:
+                files = check_manifest.get_vcs_files()
         newrepo = basetemp.join(self.rootpath.basename)
         for fn in files:
             source = self.rootpath.join(fn)
-            if source.isfile():
+            if source.islink():
+                dest = newrepo.join(fn)
+                dest.dirpath().ensure(dir=1)
+                dest.mksymlinkto(source.readlink(), absolute=True)
+            elif source.isfile():
                 dest = newrepo.join(fn)
                 dest.dirpath().ensure(dir=1)
                 source.copy(dest, mode=True)
@@ -323,16 +343,32 @@ class Checkout:
         setupdir_newrepo = newrepo.join(self.setupdir.relto(self.rootpath))
         return Exported(self.hub, setupdir_newrepo, self.setupdir)
 
+
 class Exported:
     def __init__(self, hub, rootpath, origrepo):
         self.hub = hub
         self.rootpath = rootpath
         self.origrepo = origrepo
         self.target_distdir = origrepo.join("dist")
-        python = py.path.local.sysfind("python")
-        if not python:
-            raise ValueError("could not find 'python' executable")
-        self.python = str(python)
+
+    @property
+    def python(self):
+        """
+        Find the best Python executable for invoking builds and uploads.
+        Command line option if used has priority, then a virtualenv if one
+        is activated, but otherwise falls back to sys.executable, the Python
+        under which devpi client is running.
+        """
+        python = self.hub.args.python
+        if python is None:
+            python = self._virtualenv_python()
+        if python is None:
+            python = sys.executable
+        return python
+
+    def _virtualenv_python(self):
+        if 'VIRTUAL_ENV' in os.environ:
+            return py.path.local.sysfind("python")
 
     def __str__(self):
         return "<Exported %s>" % self.rootpath
@@ -436,8 +472,6 @@ sdistformat2option = {
     "tbz": "bztar",  # bzip2'ed tar-file
 }
 
-#def getallformats():
-#    return set(sdistformat2option) + set(sdistformat2option.values())
 
 def sdistformat(format):
     """ return sdist format option. """
