@@ -1,6 +1,6 @@
 from __future__ import print_function
 from contextlib import closing
-from time import sleep
+import codecs
 import os
 import pkg_resources
 import pytest
@@ -9,24 +9,16 @@ import textwrap
 import py
 import sys
 import json
+import time
 
-from _pytest.pytester import RunResult, LineMatcher
-from devpi.main import Hub, initmain, parse_args
+from .reqmock import reqmock  # noqa
+from devpi.main import Hub, get_pluginmanager, initmain, parse_args
 from devpi_common.url import URL
-from devpi_server import __version__ as devpi_server_version
-from test_devpi_server.conftest import reqmock  # noqa
-try:
-    from test_devpi_server.conftest import simpypi, simpypiserver  # noqa
-except ImportError:
-    # when testing with older devpi-server
-    pass
 
 import subprocess
 
 print_ = py.builtin.print_
-std = py.std
 
-pytest_plugins = "pytester"
 
 def pytest_addoption(parser):
     parser.addoption("--fast", help="skip functional/slow tests", default=False,
@@ -39,6 +31,7 @@ def print_info(*args, **kwargs):
     kwargs.setdefault("file", sys.stderr)
     return py.builtin.print_(*args, **kwargs)
 
+
 class PopenFactory:
     def __init__(self, addfinalizer):
         self.addfinalizer = addfinalizer
@@ -46,68 +39,59 @@ class PopenFactory:
     def __call__(self, args, pipe=False, **kwargs):
         args = [str(x) for x in args]
         if pipe:
-            print ("$ %s [piped]" %(" ".join(args),))
+            print("$ %s [piped]" %(" ".join(args),))
             popen = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         else:
             showkwargs = " ".join(["%s=%s"] % (x,y) for x,y in kwargs.items())
-            print ("$ %s %s" %(" ".join(args), showkwargs))
+            print("$ %s %s" %(" ".join(args), showkwargs))
             popen = subprocess.Popen(args, **kwargs)
+
         def fin():
             try:
                 popen.kill()
                 popen.wait()
             except OSError:
-                print ("could not kill %s" % popen.pid)
+                print("could not kill %s" % popen.pid)
+
         self.addfinalizer(fin)
         return popen
 
+
 @pytest.fixture(scope="session")
 def Popen_session(request):
     return PopenFactory(request.addfinalizer)
 
+
 @pytest.fixture(scope="module")
 def Popen_module(request):
     return PopenFactory(request.addfinalizer)
 
+
 @pytest.fixture(scope="function")
 def Popen(request):
     return PopenFactory(request.addfinalizer)
 
-def get_pypirc_patcher(devpi):
-    hub = devpi("use")
-    user, password = hub.current.get_auth()
-    pypisubmit = hub.current.pypisubmit
-    class overwrite:
-        def __enter__(self):
-            try:
-                homedir = py.path.local._gethomedir()
-            except Exception:
-                pytest.skip("this test requires a home directory")
-            self.pypirc = pypirc = homedir.join(".pypirc")
-            if pypirc.check():
-                save = pypirc.new(basename=pypirc.basename+".save")
-                pypirc.copy(save)
-                self.saved = save
-            else:
-                self.saved = None
-            content = textwrap.dedent("""
-                [distutils]
-                index-servers = testrepo
-                [testrepo]
-                repository: %s
-                username: %s
-                password: %s
-            """ % (pypisubmit, user, password))
-            print ("patching .pypirc content:\n%s" % content)
-            pypirc.write(content)
-            return "testrepo"
 
-        def __exit__(self, *args):
-            if self.saved is None:
-                self.pypirc.remove()
-            else:
-                self.saved.copy(self.pypirc)
-    return overwrite()
+@pytest.fixture(scope="session")
+def simpypiserver():
+    from .simpypi import httpserver, SimPyPIRequestHandler
+    import threading
+    host = 'localhost'
+    port = get_open_port(host)
+    server = httpserver.HTTPServer((host, port), SimPyPIRequestHandler)
+    thread = threading.Thread(target=server.serve_forever)
+    thread.daemon = True
+    thread.start()
+    wait_for_port(host, port, 5)
+    print("Started simpypi server %s:%s" % server.server_address)
+    return server
+
+
+@pytest.fixture
+def simpypi(simpypiserver):
+    from .simpypi import SimPyPI
+    simpypiserver.simpypi = SimPyPI(simpypiserver.server_address)
+    return simpypiserver.simpypi
 
 
 def get_open_port(host):
@@ -124,44 +108,140 @@ def wait_for_port(host, port, timeout=60):
             s.settimeout(1)
             if s.connect_ex((host, port)) == 0:
                 return
-        sleep(1)
+        time.sleep(1)
         timeout -= 1
     raise RuntimeError(
         "The port %s on host %s didn't become accessible" % (port, host))
 
 
-def _liveserver(clientdir):
-    host = 'localhost'
-    port = get_open_port(host)
+def find_python3():
+    locations = [
+        "C:\\Python36-x64\\python.exe",
+        "C:\\Python36\\python.exe",
+        "C:\\Python37-x64\\python.exe",
+        "C:\\Python37\\python.exe"]
+    for location in locations:
+        if not os.path.exists(location):
+            continue
+        try:
+            output = subprocess.check_output([location, '--version'])
+            if output.strip().startswith('Python 3'):
+                return location
+        except subprocess.CalledProcessError:
+            continue
+    names = [
+        'python3.6',
+        'python3.7',
+        'python3']
+    for name in names:
+        path = py.path.local.sysfind(name)
+        if not path:
+            continue
+        path = str(path)
+        try:
+            output = subprocess.check_output([path, '--version'])
+            if output.strip().startswith('Python 3'):
+                return path
+        except subprocess.CalledProcessError:
+            continue
+    raise RuntimeError("Can't find a Python 3 executable.")
+
+
+def get_venv_script(venv_path, script_names):
+    for bindir in ('Scripts', 'bin'):
+        for script_name in script_names:
+            script = venv_path.join(bindir, script_name)
+            if script.exists():
+                return str(script)
+    else:
+        raise RuntimeError("Can't find %s in %s." % (script_names, venv_path))
+
+
+@pytest.fixture(scope="session")
+def server_executable(request, tmpdir_factory):
+    if request.config.option.fast:
+        pytest.skip("not running functional tests in --fast mode")
+    # first try installed devpi-server for quick runs during development
     path = py.path.local.sysfind("devpi-server")
-    assert path
+    if path:
+        return str(path)
+    # there is no devpi-server installed
+    python3 = find_python3()
+    # prepare environment for subprocess call
+    env = dict(os.environ)
+    env.pop("VIRTUALENV_PYTHON", None)
+    env.pop("VIRTUAL_ENV", None)
+    if sys.platform != "win32":
+        env.pop("PATH", None)
+    # create a virtualenv with Python 3
+    venv_path = tmpdir_factory.mktemp("server_venv")
+    subprocess.check_call(
+        [sys.executable, '-m', 'virtualenv', '-p', python3, str(venv_path)],
+        env=env)
+    # install devpi-server
+    venv_pip = get_venv_script(venv_path, ('pip', 'pip.exe'))
+    subprocess.check_call(
+        [venv_pip, 'install', 'devpi-server'],
+        env=env)
+    return get_venv_script(venv_path, ('devpi-server', 'devpi-server.exe'))
+
+
+@pytest.fixture(scope="session")
+def server_version(server_executable):
     try:
-        args = [
-            str(path), "--serverdir", str(clientdir), "--debug",
-            "--host", host, "--port", str(port)]
-        server_version = pkg_resources.parse_version(devpi_server_version)
-        if server_version >= pkg_resources.parse_version('4.2.0.dev'):
-            subprocess.check_call(args + ['--init'])
+        output = subprocess.check_output([server_executable, "--version"])
+        return pkg_resources.parse_version(output.decode('ascii').strip())
     except subprocess.CalledProcessError as e:
         # this won't output anything on Windows
         print(
             getattr(e, 'output', "Can't get process output on Windows"),
             file=sys.stderr)
         raise
-    p = subprocess.Popen(args)
+
+
+@pytest.fixture(scope="session")
+def indexer_backend_option(server_executable):
+    out = subprocess.check_output([server_executable, '-h'])
+    if b'--indexer-backend' in out:
+        return ['--indexer-backend', 'null']
+    return []
+
+
+def _liveserver(clientdir, indexer_backend_option, server_executable, server_version):
+    host = 'localhost'
+    port = get_open_port(host)
+    try:
+        args = [
+            "--serverdir", str(clientdir)]
+        if server_version >= pkg_resources.parse_version('5.2.0.dev'):
+            init_executable = server_executable.replace(
+                "devpi-server", "devpi-init")
+            subprocess.check_call([init_executable] + args)
+        elif server_version >= pkg_resources.parse_version('4.2.0.dev'):
+            subprocess.check_call([server_executable] + args + [
+                '--debug', '--init'])
+    except subprocess.CalledProcessError as e:
+        # this won't output anything on Windows
+        print(
+            getattr(e, 'output', "Can't get process output on Windows"),
+            file=sys.stderr)
+        raise
+    args.extend(indexer_backend_option)
+    p = subprocess.Popen([server_executable] + args + [
+        "--debug", "--host", host, "--port", str(port)])
     wait_for_port(host, port)
     return (p, URL("http://%s:%s" % (host, port)))
 
 
 @pytest.yield_fixture(scope="session")
-def url_of_liveserver(request):
+def url_of_liveserver(request, indexer_backend_option, server_executable, server_version, tmpdir_factory):
     if request.config.option.fast:
         pytest.skip("not running functional tests in --fast mode")
     if request.config.option.live_url:
         yield URL(request.config.option.live_url)
         return
-    clientdir = request.config._tmpdirhandler.mktemp("liveserver")
-    (p, url) = _liveserver(clientdir)
+    clientdir = tmpdir_factory.mktemp("liveserver")
+    (p, url) = _liveserver(clientdir, indexer_backend_option, server_executable, server_version)
     try:
         yield url
     finally:
@@ -170,11 +250,11 @@ def url_of_liveserver(request):
 
 
 @pytest.yield_fixture(scope="session")
-def url_of_liveserver2(request):
+def url_of_liveserver2(request, indexer_backend_option, server_executable, server_version, tmpdir_factory):
     if request.config.option.fast:
         pytest.skip("not running functional tests in --fast mode")
-    clientdir = request.config._tmpdirhandler.mktemp("liveserver2")
-    (p, url) = _liveserver(clientdir)
+    clientdir = tmpdir_factory.mktemp("liveserver2")
+    (p, url) = _liveserver(clientdir, indexer_backend_option, server_executable, server_version)
     try:
         yield url
     finally:
@@ -195,28 +275,148 @@ def devpi(cmd_devpi, devpi_username, url_of_liveserver):
     cmd_devpi("login", user, "--password", "123")
     cmd_devpi("index", "-c", "dev")
     cmd_devpi("use", "dev")
-    cmd_devpi.patched_pypirc = get_pypirc_patcher(cmd_devpi)
     return cmd_devpi
 
+
+def _path_parts(path):
+    path = path and str(path)  # py.path.local support
+    parts = []
+    while path:
+        folder, name = os.path.split(path)
+        if folder == path:  # root folder
+            folder, name = name, folder
+        if name:
+            parts.append(name)
+        path = folder
+    parts.reverse()
+    return parts
+
+
+def _path_join(base, *args):
+    # workaround for a py.path.local bug on Windows (`path.join('/x', abs=1)`
+    # should be py.path.local('X:\\x') where `X` is the current drive, when in
+    # fact it comes out as py.path.local('\\x'))
+    return py.path.local(base.join(*args, abs=1))
+
+
+def _filedefs_contains(base, filedefs, path):
+    """
+    whether `filedefs` defines a file/folder with the given `path`
+
+    `path`, if relative, will be interpreted relative to the `base` folder, and
+    whether relative or not, must refer to either the `base` folder or one of
+    its direct or indirect children. The base folder itself is considered
+    created if the filedefs structure is not empty.
+
+    """
+    unknown = object()
+    base = py.path.local(base)
+    path = _path_join(base, path)
+
+    path_rel_parts = _path_parts(path.relto(base))
+    for part in path_rel_parts:
+        if not isinstance(filedefs, dict):
+            return False
+        filedefs = filedefs.get(part, unknown)
+        if filedefs is unknown:
+            return False
+    return path_rel_parts or path == base and filedefs
+
+
+def create_files(base, filedefs):
+    for key, value in filedefs.items():
+        if isinstance(value, dict):
+            create_files(base.ensure(key, dir=1), value)
+        elif isinstance(value, py.builtin._basestring):
+            s = textwrap.dedent(value)
+            base.join(key).write(s)
+
+
 @pytest.fixture
-def initproj(request, tmpdir):
-    from tox._pytestplugin import initproj
-    return initproj(request, tmpdir)
+def initproj(tmpdir):
+    """Create a factory function for creating example projects.
+
+    Constructed folder/file hierarchy examples:
+
+    with `src_root` other than `.`:
+
+      tmpdir/
+          name/                  # base
+            src_root/            # src_root
+                name/            # package_dir
+                    __init__.py
+                name.egg-info/   # created later on package build
+            setup.py
+
+    with `src_root` given as `.`:
+
+      tmpdir/
+          name/                  # base, src_root
+            name/                # package_dir
+                __init__.py
+            name.egg-info/       # created later on package build
+            setup.py
+    """
+
+    def initproj_(nameversion, filedefs=None, src_root="."):
+        if filedefs is None:
+            filedefs = {}
+        if not src_root:
+            src_root = "."
+        if isinstance(nameversion, py.builtin._basestring):
+            parts = nameversion.split(str("-"))
+            if len(parts) == 1:
+                parts.append("0.1")
+            name, version = parts
+        else:
+            name, version = nameversion
+        base = tmpdir.join(name)
+        src_root_path = _path_join(base, src_root)
+        assert base == src_root_path or src_root_path.relto(
+            base
+        ), "`src_root` must be the constructed project folder or its direct or indirect subfolder"
+
+        base.ensure(dir=1)
+        create_files(base, filedefs)
+        if not _filedefs_contains(base, filedefs, "setup.py"):
+            create_files(
+                base,
+                {
+                    "setup.py": """
+                from setuptools import setup, find_packages
+                setup(
+                    name='{name}',
+                    description='{name} project',
+                    version='{version}',
+                    license='MIT',
+                    platforms=['unix', 'win32'],
+                    packages=find_packages('{src_root}'),
+                    package_dir={{'':'{src_root}'}},
+                )
+            """.format(
+                        **locals()
+                    )
+                },
+            )
+        if not _filedefs_contains(base, filedefs, src_root_path.join(name)):
+            create_files(
+                src_root_path, {name: {"__init__.py": "__version__ = {!r}".format(version)}}
+            )
+        manifestlines = [
+            "include {}".format(p.relto(base)) for p in base.visit(lambda x: x.check(file=1))
+        ]
+        create_files(base, {"MANIFEST.in": "\n".join(manifestlines)})
+        print("created project in {}".format(base))
+        base.chdir()
+        return base
+
+    return initproj_
+
 
 @pytest.fixture
 def create_and_upload(request, devpi, initproj, Popen):
     def upload(name, filedefs=None, opts=()):
         initproj(name, filedefs)
-        # we need to patch .pypirc
-        #with devpi.patched_pypirc as reponame:
-        #    popen = Popen([sys.executable, "setup.py",
-        #                   "register", "-r", reponame])
-        #    popen.communicate()
-        #    assert popen.returncode == 0
-        #    popen = Popen([sys.executable, "setup.py", "sdist", "upload",
-        #                   "-r", reponame])
-        #    popen.communicate()
-        #    assert popen.returncode == 0
         devpi("upload", *opts)
     return upload
 
@@ -225,6 +425,7 @@ def create_and_upload(request, devpi, initproj, Popen):
 def gen():
     return Gen()
 
+
 class Gen:
     def __init__(self):
         import hashlib
@@ -322,12 +523,14 @@ def ext_devpi(request, tmpdir, devpi):
         return result
     return doit
 
+
 @pytest.fixture
 def out_devpi(devpi):
     def out_devpi_func(*args, **kwargs):
+        from _pytest.pytester import RunResult
         cap = py.io.StdCaptureFD()
         cap.startall()
-        now = std.time.time()
+        now = time.time()
         try:
             try:
                 devpi(*args, **kwargs)
@@ -340,9 +543,10 @@ def out_devpi(devpi):
             raise
         print_(out)
         print_(err, file=sys.stderr)
-        return RunResult(0, out.split("\n"), None, std.time.time()-now)
+        return RunResult(0, out.split("\n"), None, time.time()-now)
     return out_devpi_func
 
+
 @pytest.fixture
 def cmd_devpi(tmpdir, monkeypatch):
     """ execute devpi subcommand in-process (with fresh init) """
@@ -350,6 +554,7 @@ def cmd_devpi(tmpdir, monkeypatch):
         print("%s: yes" % msg)
         return True
     clientdir = tmpdir.join("client")
+
     def run_devpi(*args, **kwargs):
         callargs = []
         for arg in ["devpi", "--clientdir", clientdir] + list(args):
@@ -364,7 +569,7 @@ def cmd_devpi(tmpdir, monkeypatch):
             method(hub, hub.args)
         except SystemExit as sysex:
             hub.sysex = sysex
-            if expected == None or expected < 0 or expected >= 400:
+            if expected is None or expected < 0 or expected >= 400:
                 # we expected an error or nothing, don't raise
                 pass
             else:
@@ -386,23 +591,26 @@ def cmd_devpi(tmpdir, monkeypatch):
     run_devpi.clientdir = clientdir
     return run_devpi
 
+
 @pytest.fixture
 def runproc():
     return runprocess
 
+
 def runprocess(tmpdir, cmdargs):
+    from _pytest.pytester import RunResult
     cmdargs = [str(x) for x in cmdargs]
     p1 = tmpdir.join("stdout")
     print_info("running", cmdargs, "curdir=", py.path.local())
-    with std.codecs.open(str(p1), "w", encoding="utf8") as f1:
-        now = std.time.time()
+    with codecs.open(str(p1), "w", encoding="utf8") as f1:
+        now = time.time()
         popen = subprocess.Popen(
                     cmdargs, stdout=f1, stderr=subprocess.STDOUT,
                     close_fds=(sys.platform != "win32"))
         ret = popen.wait()
-    with std.codecs.open(str(p1), "r", encoding="utf8") as f1:
+    with codecs.open(str(p1), "r", encoding="utf8") as f1:
         outerr = f1.read().splitlines()
-    return RunResult(ret, outerr, None, std.time.time()-now)
+    return RunResult(ret, outerr, None, time.time()-now)
 
 
 @pytest.fixture
@@ -410,28 +618,29 @@ def mockhtml(monkeypatch):
     def mockhtml(cache, mockurl, content):
         mockurl = cache._getsimpleurl(mockurl)
         old = cache.http.gethtml
+
         def newgethtml(url):
             if url == mockurl:
                 return content
             return old(url)
+
         monkeypatch.setattr(cache.http, "gethtml", newgethtml)
     return mockhtml
 
+
 @pytest.fixture
-def create_venv(request, testdir, monkeypatch):
+def create_venv(request, tmpdir_factory, monkeypatch):
     monkeypatch.delenv("PYTHONDONTWRITEBYTECODE", raising=False)
-    th = request.config._tmpdirhandler
-    backupenv = th.ensuretemp("venvbackup")
-    venvdir = th.ensuretemp("venv")
+    venvdir = tmpdir_factory.mktemp("venv")
+    venvinstalldir = tmpdir_factory.mktemp("inst")
+
     def do_create_venv():
-        if not venvdir.listdir():
-            assert not backupenv.listdir()
-            result = testdir.run("virtualenv", "--never-download", venvdir)
-            assert result.ret == 0
-            venvdir.copy(backupenv, mode=True)
-        else:
-            venvdir.remove()
-            backupenv.copy(venvdir, mode=True)
+        # we need to change directory, otherwise the path will become
+        # too long on windows
+        venvinstalldir.ensure_dir()
+        os.chdir(venvinstalldir.strpath)
+        subprocess.check_call([
+            "virtualenv", "--never-download", venvdir.strpath])
         # activate
         if sys.platform == "win32":
             bindir = "Scripts"
@@ -439,41 +648,49 @@ def create_venv(request, testdir, monkeypatch):
             bindir = "bin"
         monkeypatch.setenv("PATH", bindir + os.pathsep + os.environ["PATH"])
         return venvdir
+
     return do_create_venv
 
 
 @pytest.fixture
 def loghub(tmpdir):
+    from _pytest.pytester import LineMatcher
+
     class args:
         debug = True
         clientdir = tmpdir.join("clientdir")
         yes = False
         verbose = False
         settrusted = False
+
     out = py.io.TextIO()
     hub = Hub(args, file=out)
+
     def _getmatcher():
         lines = out.getvalue().split("\n")
         return LineMatcher(lines)
+
     hub._getmatcher = _getmatcher
     return hub
 
+
 @pytest.fixture(scope="session")
-def makehub(request):
-    handler = request.config._tmpdirhandler
+def makehub(tmpdir_factory):
     def mkhub(arglist):
         arglist = [str(x) for x in arglist]
-        tmp = handler.mktemp("hub")
+        tmp = tmpdir_factory.mktemp("hub")
         for x in arglist:
             if "--clientdir" in x:
                 break
         else:
             arglist.append("--clientdir=%s" % tmp)
-        args = parse_args(["devpi_"] + arglist)
+        pm = get_pluginmanager()
+        args = parse_args(["devpi_"] + arglist, pm)
         with tmp.as_cwd():
             return Hub(args)
     return mkhub
 
+
 @pytest.fixture
 def mock_http_api(monkeypatch):
     """ mock out all Hub.http_api calls and return an object
@@ -500,8 +717,10 @@ def mock_http_api(monkeypatch):
                 class R:
                     status_code = reply_data["status"]
                     reason = reply_data.get("reason", "OK")
+
                     def json(self):
                         return reply_data["json"]
+
                 return main.HTTPReply(R())
             pytest.fail("http_api call to %r is not mocked" % (url,))
 
@@ -511,4 +730,3 @@ def mock_http_api(monkeypatch):
     mockapi = MockHTTPAPI()
     monkeypatch.setattr(main.Hub, "http_api", mockapi)
     return mockapi
-
