@@ -1,13 +1,14 @@
 from io import BytesIO
+import json
 import py
 import pytest
 import requests
 import tarfile
 import time
 
-from test_devpi_server.functional import TestUserThings, TestIndexThings # noqa
+from .functional import TestUserThings, TestIndexThings # noqa
 try:
-    from test_devpi_server.functional import TestMirrorIndexThings  # noqa
+    from .functional import TestMirrorIndexThings  # noqa
 except ImportError:
     # when testing with older devpi-server
     class TestMirrorIndexThings:
@@ -15,21 +16,24 @@ except ImportError:
             pytest.skip(
                 "Couldn't import TestMirrorIndexThings from devpi server tests.")
 try:
-    from test_devpi_server.functional import TestIndexPushThings  # noqa
+    from .functional import TestIndexPushThings  # noqa
 except ImportError:
     # when testing with older devpi-server
     class TestIndexPushThings:
         def test_mirror_things(self):
             pytest.skip(
                 "Couldn't import TestIndexPushThings from devpi server tests.")
-from test_devpi_server.functional import MappMixin
+from .functional import MappMixin
+
 
 @pytest.fixture
 def mapp(request, devpi, out_devpi, tmpdir):
     return Mapp(request, devpi, out_devpi, tmpdir)
 
+
 class Mapp(MappMixin):
     _usercount = 10
+
     def __init__(self, request, devpi, out_devpi, tmpdir):
         self.devpi = devpi
         self.out_devpi = out_devpi
@@ -96,7 +100,7 @@ class Mapp(MappMixin):
     def getjson(self, path, code=200):
         result = self.out_devpi("getjson", path, code=code)
         if code == 200:
-            return py.std.json.loads(result.stdout.str())
+            return json.loads(result.stdout.str())
 
     def getindexlist(self):
         result = self.out_devpi("index", "-l")
@@ -168,7 +172,7 @@ class Mapp(MappMixin):
 
     def modify_index(self, index, indexconfig, code=200):
         import json
-        jsonfile = self.tmpdir.join("jsonfile" )
+        jsonfile = self.tmpdir.join("jsonfile")
         jsonfile.write(json.dumps(indexconfig))
         self.devpi("patchjson", "/" + index, jsonfile, code=code)
 
@@ -215,7 +219,7 @@ class Mapp(MappMixin):
             parts = line.split("acl_upload=", 1)
             if len(parts) == 2:
                 return [x for x in parts[1].split(",") if x]
-        return  []
+        return []
 
     def get_mirror_whitelist(self, code=200, indexname=None):
         indexname = self._getindexname(indexname)
@@ -260,49 +264,57 @@ def test_switch_preserves_auth(out_devpi, url_of_liveserver, url_of_liveserver2)
     import re
     result1 = out_devpi("use", url_of_liveserver)
     (url1, user1) = re.search(
-        '(https?://.+?)\s+\(logged in as (.+?)\)', result1.stdout.str()).groups()
+        r'(https?://.+?)\s+\(logged in as (.+?)\)', result1.stdout.str()).groups()
     result2 = out_devpi("use", url_of_liveserver2)
     url2 = re.search(
-        '(https?://.+?)\s+\(not logged in\)', result2.stdout.str()).group(1)
+        r'(https?://.+?)\s+\(not logged in\)', result2.stdout.str()).group(1)
     assert url2 != url1
     out_devpi("user", "-c", user1, "password=123", "email=123")
     out_devpi("login", user1, "--password", "123")
     out_devpi("index", "-c", "dev")
     result3 = out_devpi("use", "dev")
     (url3, user3) = re.search(
-        '(https?://.+?)\s+\(logged in as (.+?)\)', result3.stdout.str()).groups()
+        r'(https?://.+?)\s+\(logged in as (.+?)\)', result3.stdout.str()).groups()
     assert user3 == user1
     assert url3.startswith(url2)
     result4 = out_devpi("use", url_of_liveserver)
     (url4, user4) = re.search(
-        '(https?://.+?)\s+\(logged in as (.+?)\)', result4.stdout.str()).groups()
+        r'(https?://.+?)\s+\(logged in as (.+?)\)', result4.stdout.str()).groups()
     assert user4 == user1
     assert url4 == url1
 
 
+@pytest.fixture
+def new_user_id(gen, mapp):
+    """Create a new user id.
+
+       In case it was used to create a user, that user will be deleted on
+       fixture tear down.
+    """
+    user_id = "tmp_%s_%s" % (gen.user(), str(time.time()))
+    yield user_id
+    try:
+        mapp.delete_user(user=user_id, code=201)
+    except:  # noqa
+        # We need a bare except here, because there are exceptions from
+        # pytest and other places which don't derive from Exception and
+        # listing them all would be long and not future proof
+        pass
+
+
+@pytest.fixture
+def existing_user_id(new_user_id, mapp):
+    """Create a temporary user."""
+    mapp.create_user(
+        user=new_user_id, password=1234,
+        email=new_user_id + "@example.com")
+    return new_user_id
+
+
 class TestUserManagement:
     """ This class tests the user sub command of devpi
     """
 
-    @pytest.fixture
-    def new_user_id(self, request, gen, mapp):
-        user_id = "tmp_%s_%s"  %(gen.user(), str(time.time()))
-        def del_tmp_user():
-            try:
-                mapp.delete_user(user = user_id, code=201)
-            except:
-                pass
-        request.addfinalizer(del_tmp_user)
-        return user_id
-
-    @pytest.fixture
-    def existing_user_id(self, request, new_user_id, mapp):
-        """ Create a temporary user - will be deleted when the
-        fixture is finalized"""
-        mapp.create_user(user=new_user_id, password=1234,
-                         email=new_user_id + "@example.com", code=201)
-        return new_user_id
-
     @pytest.fixture
     def user_list(self, mapp, url_of_liveserver):
         """ This fixture gets the list of user via getjson"""
@@ -384,3 +396,23 @@ class TestUserManagement:
         """
         mapp.login_root()
         mapp.delete_user(user="root", code=403)
+
+
+class TestAddRemoveSettings:
+    def test_add_remove_index_list_setting(self, mapp, existing_user_id, server_version, url_of_liveserver):
+        from pkg_resources import parse_version
+        jsonpatch_devpi_version = parse_version("4.7.2dev")
+        if server_version < jsonpatch_devpi_version:
+            pytest.skip("devpi-server without key value parsing support")
+        mapp.login(user=existing_user_id, password="1234")
+        indexname = '%s/dev' % existing_user_id
+        indexurl = url_of_liveserver.joinpath(indexname)
+        mapp.create_index(indexname)
+        json = mapp.getjson(indexurl)
+        assert json['result']['acl_upload'] == [existing_user_id]
+        mapp.devpi("index", indexname, "acl_upload+=foo")
+        json = mapp.getjson(indexurl)
+        assert 'foo' in json['result']['acl_upload']
+        mapp.devpi("index", indexname, "acl_upload-=foo")
+        json = mapp.getjson(indexurl)
+        assert json['result']['acl_upload'] == [existing_user_id]
