@@ -1,13 +1,16 @@
-import py
 import pytest
 import json
-from devpi.main import Hub, check_output
+import textwrap
+from devpi.main import Hub
 from devpi.push import parse_target, PyPIPush, DevpiPush
+from subprocess import check_output
+
 
 def runproc(cmd):
     args = cmd.split()
     return check_output(args)
 
+
 def test_parse_target_devpi(loghub):
     class args:
         target = "user/name"
@@ -15,9 +18,10 @@ def test_parse_target_devpi(loghub):
     res = parse_target(loghub, args)
     assert isinstance(res, DevpiPush)
 
+
 def test_parse_target_pypi(tmpdir, loghub):
     p = tmpdir.join("pypirc")
-    p.write(py.std.textwrap.dedent("""
+    p.write(textwrap.dedent("""
         [distutils]
         index-servers = whatever
 
@@ -26,10 +30,12 @@ def test_parse_target_pypi(tmpdir, loghub):
         username: test
         password: testp
     """))
+
     class args:
         target = "pypi:whatever"
         pypirc = str(p)
         index = None
+
     res = parse_target(loghub, args)
     assert isinstance(res, PyPIPush)
     assert res.user == "test"
@@ -39,7 +45,7 @@ def test_parse_target_pypi(tmpdir, loghub):
 
 def test_parse_target_pypi_default_repository(tmpdir, loghub):
     p = tmpdir.join("pypirc")
-    p.write(py.std.textwrap.dedent("""
+    p.write(textwrap.dedent("""
         [distutils]
         index-servers = whatever
 
@@ -47,15 +53,17 @@ def test_parse_target_pypi_default_repository(tmpdir, loghub):
         username: test
         password: testp
     """))
+
     class args:
         target = "pypi:whatever"
         pypirc = str(p)
         index = None
+
     res = parse_target(loghub, args)
     assert isinstance(res, PyPIPush)
     assert res.user == "test"
     assert res.password == "testp"
-    assert res.posturl == "https://www.python.org/pypi"
+    assert res.posturl == "https://upload.pypi.org/legacy/"
 
 
 def test_push_devpi(loghub, monkeypatch, mock_http_api):
@@ -83,11 +91,13 @@ def test_push_devpi_index_option(loghub, monkeypatch, mock_http_api):
 
 
 @pytest.mark.parametrize("spec", ("pkg==1.0", "pkg-1.0"))
-def test_main_push_pypi(monkeypatch, tmpdir, spec):
+def test_main_push_pypi(capsys, monkeypatch, tmpdir, spec):
     from devpi.push import main
     l = []
+
     def mypost(method, url, data, headers, auth=None, cert=None, verify=None):
         l.append((method, url, data))
+
         class r:
             status_code = 201
             reason = "created"
@@ -108,7 +118,7 @@ def test_main_push_pypi(monkeypatch, tmpdir, spec):
     monkeypatch.setattr(hub.http, "request", mypost)
     hub.current.reconfigure(dict(index="/some/index"))
     p = tmpdir.join("pypirc")
-    p.write(py.std.textwrap.dedent("""
+    p.write(textwrap.dedent("""
         [distutils]
         index-servers = whatever
 
@@ -117,6 +127,7 @@ def test_main_push_pypi(monkeypatch, tmpdir, spec):
         username: test
         password: testp
     """))
+
     class args:
         pypirc = str(p)
         target = "pypi:whatever"
@@ -127,19 +138,34 @@ def test_main_push_pypi(monkeypatch, tmpdir, spec):
     assert len(l) == 1
     method, url, data = l[0]
     assert url == hub.current.index
-    req = py.std.json.loads(data)
+    req = json.loads(data)
     assert req["name"] == "pkg"
     assert req["version"] == "1.0"
     assert req["posturl"] == "http://anotherserver"
     assert req["username"] == "test"
     assert req["password"] == "testp"
 
+    class args:
+        pypirc = str(p)
+        target = "pypi:notspecified"
+        pkgspec = spec
+        index = None
+
+    (out, err) = capsys.readouterr()
+    with pytest.raises(SystemExit):
+        main(hub, args)
+    (out, err) = capsys.readouterr()
+    assert "Error while trying to read section 'notspecified'" in out
+    assert "KeyError" in out
+
 
 def test_fail_push(monkeypatch, tmpdir):
     from devpi.push import main
     l = []
+
     def mypost(method, url, data, headers, auth=None, cert=None, verify=None):
         l.append((method, url, data))
+
         class r:
             status_code = 500
             reason = "Internal Server Error"
@@ -148,8 +174,10 @@ def test_fail_push(monkeypatch, tmpdir):
             ))
             headers = {"content-type": "application/json"}
             _json = json.loads(content)
+
             class request:
                 method = ''
+
         r.url = url
         r.request.method = method
 
@@ -163,7 +191,7 @@ def test_fail_push(monkeypatch, tmpdir):
     monkeypatch.setattr(hub.http, "request", mypost)
     hub.current.reconfigure(dict(index="/some/index"))
     p = tmpdir.join("pypirc")
-    p.write(py.std.textwrap.dedent("""
+    p.write(textwrap.dedent("""
         [distutils]
         index-servers = whatever
 
@@ -172,6 +200,7 @@ def test_fail_push(monkeypatch, tmpdir):
         username: test
         password: testp
     """))
+
     class args:
         pypirc = str(p)
         target = "pypi:whatever"
