@@ -1,10 +1,15 @@
+import os
 import subprocess
+import py
 import pytest
 import sys
 import tox
-from devpi.test import *
+from devpi_common.viewhelp import ViewLinkStore
+from devpi.test import DevIndex
+from devpi.test import find_sdist_and_wheels
+from devpi.test import prepare_toxrun_args
+from devpi.test import post_tox_json_report
 
-pytest_plugins = "pytester"
 
 def test_post_tox_json_report(loghub, mock_http_api):
     mock_http_api.set("http://devpi.net", 200, result={})
@@ -15,6 +20,7 @@ def test_post_tox_json_report(loghub, mock_http_api):
         *success*
     """)
 
+
 def test_post_tox_json_report_error(loghub, mock_http_api):
     mock_http_api.set("http://devpi.net/+tests", 404)
     post_tox_json_report(loghub, "http://devpi.net/+tests", {"hello": "123"})
@@ -23,12 +29,14 @@ def test_post_tox_json_report_error(loghub, mock_http_api):
         *could not post*http://devpi.net/+tests*
     """)
 
+
 @pytest.fixture
 def pseudo_current():
     class Current:
         simpleindex = "http://pseudo/user/index/"
     return Current
 
+
 def contains_sublist(list1, sublist):
     len_sublist = len(sublist)
     assert len_sublist <= len(list1)
@@ -37,6 +45,7 @@ def contains_sublist(list1, sublist):
             return True
     return False
 
+
 def test_passthrough_args_toxargs(makehub, tmpdir, pseudo_current):
     hub = makehub(["test", "--tox-args", "-- -x", "somepkg"])
     index = DevIndex(hub, tmpdir, pseudo_current)
@@ -57,7 +66,7 @@ def test_index_option(create_and_upload, devpi, monkeypatch, out_devpi):
     # remember username
     out = out_devpi("use")
     (url, user) = re.search(
-        '(https?://.+?)\s+\(logged in as (.+?)\)', out.stdout.str()).groups()
+        r'(https?://.+?)\s+\(logged in as (.+?)\)', out.stdout.str()).groups()
 
     # go to other index
     devpi("use", "root/pypi")
@@ -88,7 +97,7 @@ def test_download_and_unpack(makehub, tmpdir, pseudo_current, monkeypatch,
                 self.content = content
 
         def __init__(self):
-                self.last_get = None
+            self.last_get = None
 
         def get(self, *args, **kwargs):
             self.last_get = (args, kwargs)
@@ -141,6 +150,7 @@ def test_toxini(makehub, tmpdir, pseudo_current):
     args = index.get_tox_args(unpack_path=tmpdir)
     assert contains_sublist(args, ["-c", str(toxini)])
 
+
 def test_passthrough_args_env(makehub, tmpdir, pseudo_current):
     hub = makehub(["test", "-epy27", "somepkg"])
     index = DevIndex(hub, tmpdir, pseudo_current)
@@ -148,18 +158,20 @@ def test_passthrough_args_env(makehub, tmpdir, pseudo_current):
     args = index.get_tox_args(unpack_path=tmpdir)
     assert contains_sublist(args, ["-epy27"])
 
+
 def test_no_detox(makehub, tmpdir, pseudo_current):
     hub = makehub(["test", "-epy27", "somepkg"])
     index = DevIndex(hub, tmpdir, pseudo_current)
     runner = index.get_tox_runner()
     assert runner == tox.cmdline
 
+
 def test_detox(makehub, tmpdir, pseudo_current):
-    detox_main = pytest.importorskip("detox.main")
+    pytest.importorskip("detox")
     hub = makehub(["test", "--detox", "-epy27", "somepkg"])
     index = DevIndex(hub, tmpdir, pseudo_current)
     runner = index.get_tox_runner()
-    assert runner == detox_main.main
+    assert 'detox' in runner.__module__
 
 
 def test_fallback_ini(makehub, tmpdir, pseudo_current):
@@ -172,6 +184,7 @@ def test_fallback_ini(makehub, tmpdir, pseudo_current):
     args = index.get_tox_args(unpack_path=tmpdir)
     assert contains_sublist(args, ["-c", str(p2)])
 
+
 class TestWheel:
     def test_find_wheels_and_sdist(self, loghub):
         vl = ViewLinkStore("http://something/index", {"+links": [
@@ -227,6 +240,7 @@ class TestWheel:
         assert wheel_links[0].basename.endswith("py2-none-any.whl")
         assert 'only universal wheels' not in '\n'.join(loghub._getmatcher().lines)
 
+    @pytest.mark.skipif("config.option.fast")
     def test_prepare_toxrun_args(self, loghub, pseudo_current, tmpdir, reqmock, initproj):
         # XXX this test was a bit hard to setup and is also somewhat covered by
         # the below wheel functional test so unclear if it's worth to
@@ -257,6 +271,7 @@ class TestWheel:
         assert wheel1[0].basename == "prep1-1.0-py2.py3-none-any.whl"
         assert str(wheel1[1].path_unpacked).endswith(wheel1[0].basename)
 
+    @pytest.mark.skipif("config.option.fast")
     def test_prepare_toxrun_args2(self, loghub, pseudo_current, tmpdir, reqmock, initproj):
         # basically the same test as above, but it's testing the unpack
         # path for packages that have an underscore in the name
@@ -286,6 +301,7 @@ class TestWheel:
         assert wheel1[0].basename == "prep_under-1.0-py2.py3-none-any.whl"
         assert str(wheel1[1].path_unpacked).endswith(wheel1[0].basename)
 
+    @pytest.mark.skipif("config.option.fast")
     def test_prepare_toxrun_args_select(self, loghub, pseudo_current, tmpdir, reqmock, initproj):
         # test that we can explicitly select a non universal wheel
         pyver = "py%s" % sys.version_info[0]
@@ -313,7 +329,7 @@ class TestWheel:
 
     def test_wheels_and_sdist(self, out_devpi, create_and_upload):
         create_and_upload("exa-1.0", filedefs={
-           "tox.ini": """
+            "tox.ini": """
               [testenv]
               commands = python -c "print('ok')"
             """,
@@ -336,12 +352,10 @@ class TestWheel:
 
 
 class TestFunctional:
-    @pytest.mark.xfail(reason="output capturing for devpi calls")
-    def test_main_nopackage(self, out_devpi):
-        result = out_devpi("test", "--debug", "notexists73", ret=1)
-        result.fnmatch_lines([
-            "*could not find/receive*",
-        ])
+    def test_main_nopackage(self, capfd, out_devpi):
+        out_devpi("test", "--debug", "notexists73", ret=1)
+        (out, err) = capfd.readouterr()
+        assert "could not find/receive links for notexists73" in out
 
     def test_main_example(self, out_devpi, create_and_upload):
         create_and_upload("exa-1.0", filedefs={
