@@ -1,18 +1,26 @@
-
-import os, sys
 import json
+import os
 import py
 import pytest
-from devpi.upload import *
+import re
+import sys
+from devpi.upload import Checkout
+from devpi.upload import find_parent_subpath
+from devpi.upload import filter_latest
+from devpi.upload import get_pkginfo
+from devpi.upload import main
+from devpi.upload import read_setupcfg
 from textwrap import dedent
 from devpi_common.metadata import splitbasename
 from devpi_common.viewhelp import ViewLinkStore
-from devpi.main import check_output
+from subprocess import check_output
+
 
 @pytest.fixture
 def datadir():
     return py.path.local(__file__).dirpath("data")
 
+
 def runproc(cmd):
     args = cmd.split()
     path0 = args[0]
@@ -22,6 +30,7 @@ def runproc(cmd):
             pytest.skip("%r not found" % args[0])
     return check_output([str(path0)] + args[1:])
 
+
 @pytest.fixture
 def uploadhub(request, tmpdir):
     from devpi.main import initmain
@@ -36,6 +45,7 @@ def uploadhub(request, tmpdir):
 #    monkeypatch.setattr(extpypi.ExtDB, "get_releaselinks", lambda *args: 404)
 
 
+@pytest.mark.skipif("config.option.fast")
 class TestCheckout:
     @pytest.fixture(autouse=True)
     def no_sys_executable(self, monkeypatch):
@@ -51,14 +61,18 @@ class TestCheckout:
         return repo.join(setupdir_rel)
 
     @pytest.fixture(scope="class", params=["hg", "git"])
-    def repo(self, request, setupdir_rel):
-        repo = request.config._tmpdirhandler.mktemp("repo", numbered=True)
+    def repo(self, request, setupdir_rel, tmpdir_factory):
+        repo = tmpdir_factory.mktemp("repo", numbered=True)
         setupdir = repo.ensure_dir(setupdir_rel)
         file = setupdir.join("file")
         file.write("hello")
+        link = setupdir.join("link")
         setup_path = setupdir.ensure("setup.py")
         if not sys.platform.startswith("win"):
             setup_path.chmod(int("0777", 8))
+            link.mksymlinkto("..", absolute=True)
+        else:
+            link.write("no symlinks on windows")
 
         # this is a test for issue154 although we actually don't really
         # need to test the vcs-exporting code much since we started
@@ -72,7 +86,7 @@ class TestCheckout:
                 pytest.skip("'hg' command not found")
             with repo.as_cwd():
                 runproc("hg init")
-                runproc("hg add {0}/file {0}/setup.py".format(setupdir_rel))
+                runproc("hg add {0}/file {0}/link {0}/setup.py".format(setupdir_rel))
                 runproc("hg add {0}/file {0}/{1}".format(setupdir_rel,
                                                          unicode_fn))
                 runproc("hg commit --config ui.username=whatever -m message")
@@ -83,7 +97,7 @@ class TestCheckout:
             runproc("git init")
             runproc("git config user.email 'you@example.com'")
             runproc("git config user.name 'you'")
-            runproc("git add {0}/file {0}/setup.py".format(setupdir_rel))
+            runproc("git add {0}/file {0}/link {0}/setup.py".format(setupdir_rel))
             runproc("git add {0}/file {0}/{1}".format(setupdir_rel,
                                                       unicode_fn))
             runproc("git commit -m message")
@@ -95,6 +109,9 @@ class TestCheckout:
         newrepo = tmpdir.mkdir("newrepo")
         result = checkout.export(newrepo)
         assert result.rootpath.join("file").check()
+        assert result.rootpath.join("link").check()
+        if not sys.platform.startswith("win"):
+            assert result.rootpath.join("link").readlink() == ".."
         assert result.rootpath == newrepo.join(repo.basename).join(
             repo.bestrelpath(setupdir))
         # ensure we also copied repo meta info
@@ -111,10 +128,12 @@ class TestCheckout:
         newrepo = tmpdir.mkdir("newrepo")
         result = checkout.export(newrepo)
         assert result.rootpath.join("file").check()
+        assert result.rootpath.join("link").check()
         p = result.rootpath.join("setup.py")
         assert p.exists()
         if not sys.platform.startswith("win"):
             assert p.stat().mode & int("0777", 8) == int("0777", 8)
+            assert result.rootpath.join("link").readlink() == '..'
         assert result.rootpath == newrepo.join(setupdir.basename)
 
     def test_vcs_export_disabled(self, uploadhub, setupdir,
@@ -161,11 +180,13 @@ class TestCheckout:
         # is called with the exported directory instead of he original
         l = []
         old_popen_output = exported.hub.popen_output
+
         def mock_popen_output(args, **kwargs):
             if "build_sphinx" in args:
                 l.append(kwargs)
             else:
                 return old_popen_output(args, **kwargs)
+
         exported.hub.popen_output = mock_popen_output
         # now we can make the call
         exported.setup_build_docs()
@@ -187,6 +208,7 @@ def test_setup_build_formats_setupcfg(uploadhub, tmpdir):
     assert cfg.get("no-vcs") == "1"
     assert cfg.get("setupdir-only") == "1"
 
+
 def test_setup_build_formats_setupcfg_nosection(uploadhub, tmpdir):
     tmpdir.join("setup.cfg").write(dedent("""
         [bdist_wheel]
@@ -206,6 +228,7 @@ def test_parent_subpath(tmpdir):
     pytest.raises(ValueError, lambda: find_parent_subpath(tmpdir, "poiqel123"))
 
 
+@pytest.mark.skipif("config.option.fast")
 def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
     class Session:
         posts = []
@@ -290,7 +313,7 @@ class TestUploadFunctional:
             file_upload of {projname_version}.zip*
             """.format(projname_version=projname_version))
 
-        print ("*"*80)
+        print("*"*80)
         out = out_devpi("upload", "--formats", "sdist.zip,bdist_wheel",
                         code=[200, 200, 200, 200])
         out.stdout.fnmatch_lines_random("""
@@ -302,13 +325,14 @@ class TestUploadFunctional:
 
         # remember username
         out = out_devpi("use")
-        user = re.search('\(logged in as (.+?)\)', out.stdout.str()).group(1)
+        user = re.search(r'\(logged in as (.+?)\)', out.stdout.str()).group(1)
 
         # go to other index
         out = out_devpi("use", "root/pypi")
         out.stdout.fnmatch_lines_random("current devpi index*/root/pypi*")
         out = out_devpi("upload", "--dry-run")
-        out.stdout.fnmatch_lines_random("no pypisubmit endpoint available for*")
+        out.stdout.fnmatch_lines_random("*does not support upload.")
+        out.stdout.fnmatch_lines_random("*it is a mirror.")
 
         # --index option
         out = out_devpi("upload", "--index", "%s/dev" % user, "--dry-run")
@@ -381,7 +405,6 @@ class TestUploadFunctional:
         assert links["doczip"] == "%s.doc.zip" % name_version_str
 
 
-
 def test_getpkginfo(datadir):
     info = get_pkginfo(datadir.join("dddttt-0.1.dev45-py27-none-any.whl"))
     assert info.name == "dddttt"
@@ -390,6 +413,7 @@ def test_getpkginfo(datadir):
     assert info.name == "ddd"
     assert info.version == "1.0"
 
+
 def test_filter_latest():
     class PkgInfo(object):
         def __init__(self, path):
@@ -405,5 +429,3 @@ def test_filter_latest():
     filtered = d[path]
     assert filtered.name == 'test-abc'
     assert filtered.version == u'0.10'
-
-
