@@ -1,6 +1,6 @@
 from devpi.use import BuildoutCfg, DistutilsCfg, PipCfg
 from devpi.use import Current, PersistentCurrent
-from devpi.use import parse_keyvalue_spec, out_index_list
+from devpi.use import get_keyvalues, out_index_list
 from devpi_common.url import URL
 import pytest
 import re
@@ -19,10 +19,12 @@ def test_ask_confirm(makehub, monkeypatch):
     monkeypatch.setattr(devpi.main, "raw_input", lambda msg: l.pop())
     assert hub.ask_confirm("hello")
 
+
 def test_ask_confirm_delete_args_yes(makehub):
     hub = makehub(["remove", "-y", "whatever"])
     assert hub.ask_confirm("hello")
 
+
 class TestUnit:
     def test_write_and_read(self, tmpdir):
         path=tmpdir.join("current")
@@ -58,6 +60,7 @@ class TestUnit:
         assert current.always_setcfg
         assert current.simpleindex == "/index2"
 
+    @pytest.mark.skipif("config.option.fast")
     def test_use_list_doesnt_write(self, tmpdir, cmd_devpi, mock_http_api):
         import time
         mock_http_api.set(
@@ -152,8 +155,10 @@ class TestUnit:
     def test_use_with_nonexistent_domain(self, capfd, cmd_devpi, Exc,
                                          monkeypatch):
         from requests.sessions import Session
+
         def raise_(*args, **kwargs):
             raise Exc("qwe")
+
         monkeypatch.setattr(Session, "request", raise_)
         cmd_devpi("use", "http://qlwkejqlwke", code=-1)
         out, err = capfd.readouterr()
@@ -348,7 +353,7 @@ class TestUnit:
         cmd_devpi("use", "--venv=-")
 
         # test via env for virtualenvwrapper
-        monkeypatch.setenv("WORKON_HOME", venvdir.dirpath())
+        monkeypatch.setenv("WORKON_HOME", venvdir.dirpath().strpath)
         hub = cmd_devpi("use", "--venv=%s" % venvdir.basename)
         assert hub.current.venvdir == venvdir
 
@@ -356,9 +361,9 @@ class TestUnit:
         cmd_devpi("use", "--venv=-")
 
         # test via env for activated venv
-        monkeypatch.setenv("VIRTUAL_ENV", venvdir)
+        monkeypatch.setenv("VIRTUAL_ENV", venvdir.strpath)
         hub = cmd_devpi("use")
-        assert hub.current.venvdir == None, \
+        assert hub.current.venvdir is None, \
             "When --venv is not given, hub.current shouldn't be set"
         res = out_devpi("use")
         res.stdout.fnmatch_lines("*venv*%s" % venvdir)
@@ -376,7 +381,7 @@ class TestUnit:
 
     def test_venv_setcfg(self, mock_http_api, cmd_devpi, tmpdir, monkeypatch):
         from devpi.use import vbin
-        monkeypatch.setenv("HOME", tmpdir.join('home'))
+        monkeypatch.setenv("HOME", tmpdir.join('home').strpath)
         monkeypatch.setattr(PipCfg, "pip_conf_name", "pip.cfg")
         monkeypatch.setattr(DistutilsCfg, "default_location",
                             tmpdir.join("dist.cfg"))
@@ -404,16 +409,16 @@ class TestUnit:
         venv_pip_config = venvdir.join("pip.cfg")
         assert venv_pip_config.exists()
         content = venv_pip_config.read()
-        assert len(re.findall("index_url\s*=\s*%s" % index, content)) == 1
+        assert len(re.findall(r"index_url\s*=\s*%s" % index, content)) == 1
         result = re.findall(
-            "\[search\].*index\s*=\s*%s" % index.replace('simple', ''), content, flags=re.DOTALL)
+            r"\[search\].*index\s*=\s*%s" % index.replace('simple', ''), content, flags=re.DOTALL)
         assert len(result) == 1
         result = result[0].splitlines()
         assert len(result) == 2
 
     def test_active_venv_setcfg(self, mock_http_api, cmd_devpi, tmpdir, monkeypatch):
         from devpi.use import vbin
-        monkeypatch.setenv("HOME", tmpdir.join('home'))
+        monkeypatch.setenv("HOME", tmpdir.join('home').strpath)
         monkeypatch.setattr(PipCfg, "pip_conf_name", "pip.cfg")
         monkeypatch.setattr(DistutilsCfg, "default_location",
                             tmpdir.join("dist.cfg"))
@@ -431,7 +436,7 @@ class TestUnit:
         venvdir = tmpdir
         venvdir.ensure(vbin, dir=1)
         monkeypatch.chdir(tmpdir)
-        monkeypatch.setenv("VIRTUAL_ENV", venvdir)
+        monkeypatch.setenv("VIRTUAL_ENV", venvdir.strpath)
         index = "http://world/simple"
         cmd_devpi("use", "--set-cfg", index)
 
@@ -446,9 +451,9 @@ class TestUnit:
         ('https', ''),
         ('http', 'foo:bar@'),
         ('https', 'foo:bar@')])
-    def test_main_setcfg(self, scheme, basic_auth, mock_http_api, cmd_devpi, tmpdir, monkeypatch):
+    def test_main_setcfg(self, scheme, basic_auth, capfd, mock_http_api, cmd_devpi, tmpdir, monkeypatch):
         monkeypatch.delenv("VIRTUAL_ENV", raising=False)
-        monkeypatch.setenv("HOME", tmpdir.join('home'))
+        monkeypatch.setenv("HOME", tmpdir.join('home').strpath)
         monkeypatch.setattr(PipCfg, "pip_conf_name", "pip.cfg")
         monkeypatch.setattr(DistutilsCfg, "default_location",
                             tmpdir.join("dist.cfg"))
@@ -467,13 +472,17 @@ class TestUnit:
         cmd_devpi("use", "--set-cfg", "%s://%sworld" % (scheme, basic_auth))
         # run twice to find any issues where lines are added more than once
         cmd_devpi("use", "--set-cfg", "%s://%sworld" % (scheme, basic_auth))
+        if '@' in basic_auth:
+            (out, err) = capfd.readouterr()
+            assert basic_auth not in out
+            assert ':****@' in out
         assert PipCfg().default_location.exists()
         content = PipCfg().default_location.read()
         assert len(
-            re.findall("index_url\s*=\s*%s://%sworld/simple" % (
+            re.findall(r"index_url\s*=\s*%s://%sworld/simple" % (
                 scheme, basic_auth), content)) == 1
         result = re.findall(
-            "\[search\].*index\s*=\s*%s://%sworld/" % (
+            r"\[search\].*index\s*=\s*%s://%sworld/" % (
                 scheme, basic_auth), content, flags=re.DOTALL)
         assert len(result) == 1
         result = result[0].splitlines()
@@ -481,12 +490,12 @@ class TestUnit:
         assert DistutilsCfg.default_location.exists()
         content = DistutilsCfg.default_location.read()
         assert len(
-            re.findall("index_url\s*=\s*%s://%sworld/simple" % (
+            re.findall(r"index_url\s*=\s*%s://%sworld/simple" % (
                 scheme, basic_auth), content)) == 1
         assert BuildoutCfg.default_location.exists()
         content = BuildoutCfg.default_location.read()
         assert len(
-            re.findall("index\s*=\s*%s://%sworld/simple" % (
+            re.findall(r"index\s*=\s*%s://%sworld/simple" % (
                 scheme, basic_auth), content)) == 1
         hub = cmd_devpi("use", "--always-set-cfg=yes")
         assert hub.current.always_setcfg
@@ -498,32 +507,39 @@ class TestUnit:
                 scheme, basic_auth))
         content = PipCfg().default_location.read()
         assert len(
-            re.findall("trusted-host\s*=\s*world", content)) == 1
+            re.findall(r"trusted-host\s*=\s*world", content)) == 1
         hub = cmd_devpi("use", "--always-set-cfg=yes", "--pip-set-trusted=yes")
         assert hub.current.settrusted
         hub = cmd_devpi("use", "--always-set-cfg=no", "--pip-set-trusted=no")
         assert not hub.current.settrusted
 
 
+def test_getparse_keyvalues_invalid():
+    with pytest.raises(ValueError):
+        get_keyvalues(["hello123"])
+
+
 @pytest.mark.parametrize("input expected".split(), [
     (["hello=123", "world=42"], dict(hello="123", world="42")),
-    (["hello=123=1"], dict(hello="123=1"))
-    ])
-def test_parse_keyvalue_spec(input, expected):
-    result = parse_keyvalue_spec(input, "hello world".split())
-    assert result == expected
+    (["hello=123=1"], dict(hello="123=1")),
+    (["hello=1", "hello=2"], dict(hello="2")),
+    (["hello+=1"], {"hello+": "1"}),
+    (["hello-=1"], {"hello-": "1"})])
+def test_getparse_keyvalues_kvdict(input, expected):
+    result = get_keyvalues(input)
+    assert result.kvdict == expected
 
-def test_parse_keyvalue_spec_unknown_key():
-    pytest.raises(KeyError, lambda: parse_keyvalue_spec(["hello=3"], ["some"]))
 
 def test_user_no_index(loghub):
     out_index_list(loghub, {"user": {"username": "user"}})
 
+
 def test_pipcfg_default_location(tmpdir, monkeypatch):
     path = PipCfg().path
-    monkeypatch.setenv('PIP_CONFIG_FILE', tmpdir.join("cfg"))
+    monkeypatch.setenv('PIP_CONFIG_FILE', tmpdir.join("cfg").strpath)
     assert path != PipCfg().path
 
+
 class TestCfgParsing:
     @pytest.fixture(scope="class", params=[DistutilsCfg, PipCfg, BuildoutCfg])
     def cfgclass(self, request):
